@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #ifndef CONVERT_H
@@ -25,7 +26,7 @@
 
 /*-- time.c --*/
 gpg_error_t _ksba_asntime_to_iso (const char *buffer, size_t length,
-                                ksba_isotime_t timebuf);
+                                  int is_utctime, ksba_isotime_t timebuf);
 gpg_error_t _ksba_assert_time_format (const ksba_isotime_t atime);
 void _ksba_copy_time (ksba_isotime_t d, const ksba_isotime_t s);
 int _ksba_cmp_time (const ksba_isotime_t a, const ksba_isotime_t b);
